@@ -6,6 +6,7 @@ from typing import *
 from cognite.seismic._api.api import API
 from cognite.seismic.data_classes.api_types import Survey
 from cognite.seismic.data_classes.errors import SeismicServiceError
+from cognite.seismic.protos.types_pb2 import CRS, CoverageParameters
 from grpc import StatusCode
 
 if not os.getenv("READ_THE_DOCS"):
@@ -21,19 +22,29 @@ class SurveyV1API(API):
         super().__init__(metadata=metadata, query=query)
         self.v0_survey_api = v0_survey_api
 
-    def list(self, list_seismics: bool = False, list_seismic_stores: bool = False, include_metadata: bool = False):
+    def list(
+        self,
+        list_seismics: bool = False,
+        list_seismic_stores: bool = False,
+        include_metadata: bool = False,
+        crs: Optional[str] = None,
+        in_wkt: Optional[bool] = None,
+    ):
         """List all the surveys.
+        Provide either crs or in_wkt to get surveys' coverage.
 
         Args:
             list_seismics (bool): true if the seismics ids from the surveys should be listed.
             list_seismic_stores (bool): true if seismic stores ids from the surveys should be listed
             include_metadata (bool): true if metadata should be included in the response.
+            crs(str): the crs in which the surveys' coverage is returned, default is original survey crs
+            in_wkt(bool): surveys' coverage format, set to true if wkt format is needed, default is geojson format
 
         Returns:
             List[Survey]: the requested surveys and their files (if requested).
         """
 
-        return self._search_internal([], list_seismics, list_seismic_stores, include_metadata)
+        return self._search_internal([], list_seismics, list_seismic_stores, include_metadata, crs, in_wkt)
 
     def search(
         self,
@@ -42,8 +53,11 @@ class SurveyV1API(API):
         list_seismics: bool = False,
         list_seismic_stores: bool = False,
         include_metadata: bool = False,
+        crs: Optional[str] = None,
+        in_wkt: Optional[bool] = None,
     ):
         """Search for subset of surveys.
+        Provide either crs or in_wkt to get surveys' coverage.
 
         Args:
             survey_name_substring (str): find surveys whose name contains this substring
@@ -51,6 +65,8 @@ class SurveyV1API(API):
             list_seismics (bool): true if the seismics ids from the surveys should be listed.
             list_seismic_stores (bool): true if seismic stores ids from the surveys should be listed
             include_metadata (bool): true if metadata should be included in the response.
+            crs(str): the crs in which the surveys' coverage is returned, default is original survey crs
+            in_wkt(bool): surveys' coverage format, set to true if wkt format is needed, default is geojson format
 
         Returns:
             List[Survey]: the requested surveys and their files (if requested).
@@ -64,7 +80,7 @@ class SurveyV1API(API):
         if survey_external_id_substring is not None:
             search_specs.append(SearchSpec(external_id_substring=survey_external_id_substring))
 
-        return self._search_internal(search_specs, list_seismics, list_seismic_stores, include_metadata)
+        return self._search_internal(search_specs, list_seismics, list_seismic_stores, include_metadata, crs, in_wkt)
 
     def get(
         self,
@@ -74,9 +90,12 @@ class SurveyV1API(API):
         list_seismics: bool = False,
         list_seismic_stores: bool = False,
         include_metadata: bool = False,
+        crs: Optional[str] = None,
+        in_wkt: Optional[bool] = None,
     ):
         """
         Get a survey by either id, external_id or name.
+        Provide either crs or in_wkt to get survey coverage.
 
         Args:
             survey_id (str, optional): survey id.
@@ -85,6 +104,8 @@ class SurveyV1API(API):
             list_seismics (bool): true if the ids of seismics from this survey should be listed.
             list_seismic_stores (bool): true if the ids of seismic stores from this survey should be listed.
             include_metadata (bool): true if metadata should be included in the response.
+            crs(str): the crs in which the survey coverage is returned, default is original survey crs
+            in_wkt(bool): survey coverage format, set to true if wkt format is needed, default is geojson format
 
         Returns:
             Survey: the requested survey, its seismics, seismic stores and metadata (if requested).
@@ -100,7 +121,7 @@ class SurveyV1API(API):
         else:
             search_spec = SearchSpec(name=survey_name)
 
-        result = self._search_internal([search_spec], list_seismics, list_seismic_stores, include_metadata)
+        result = self._search_internal([search_spec], list_seismics, list_seismic_stores, include_metadata, crs, in_wkt)
         if len(result) == 0:
             raise SeismicServiceError(StatusCode.NOT_FOUND, "survey not found")
         else:
@@ -151,12 +172,21 @@ class SurveyV1API(API):
         list_seismics: bool = False,
         list_seismic_stores: bool = False,
         include_metadata: bool = False,
+        crs: Optional[str] = None,
+        in_wkt: Optional[bool] = None,
     ):
+        coverageParamCrs = CRS(crs=crs) if crs is not None else None
+        coverageParams = (
+            CoverageParameters(crs=coverageParamCrs, in_wkt=in_wkt)
+            if coverageParamCrs is not None or in_wkt is not None
+            else None
+        )
         request = SearchSurveysRequest(
             surveys=search_specs,
             list_seismic_ids=list_seismics,
             list_seismic_store_ids=list_seismic_stores,
             include_metadata=include_metadata,
+            include_coverage=coverageParams,
         )
         return [
             Survey.from_proto(survey_proto)
