@@ -1,8 +1,10 @@
 import json
+import math
 import os
-from typing import List, Mapping, Optional, Tuple, Union
+from typing import Iterable, List, Mapping, Optional, Tuple, Union
 
-from cognite.seismic._api.utility import MaybeString
+import numpy as np
+from cognite.seismic._api.utility import Direction, LineRange, MaybeString
 
 if not os.getenv("READ_THE_DOCS"):
     from cognite.seismic.protos.types_pb2 import CRS as CRSProto
@@ -55,51 +57,142 @@ class Coordinate:
 
 
 class Trace:
-    """Represents a seismic trace with a single inline / crossline coordinate.
+    """Represents a seismic trace with a single inline / xline coordinate.
 
     Attributes:
         trace_header (bytes): The raw trace header.
         inline (int): The inline number
-        crossline (int): The crossline number
+        xline (int): The xline number
         trace (List[float]): The trace values
         coordinate (:py:class:`Coordinate`): The coordinate of the trace
     """
 
     trace_header: bytes
     inline: int
-    crossline: int
+    xline: int
     trace: List[float]
     coordinate: Coordinate
 
-    def __init__(self, trace_header, inline, crossline, trace, coordinate):
+    def __init__(self, trace_header, inline, xline, trace, coordinate):
         self.trace_header = trace_header
         self.inline = inline
-        self.crossline = crossline
+        self.xline = xline
         self.trace = trace
         self.coordinate = coordinate
 
     def __repr__(self):
-        return f"Trace<inline: {self.inline}, crossline: {self.crossline}, coordinates: {str(self.coordinate)}, trace: {self.trace}>"
+        return f"Trace<inline: {self.inline}, xline: {self.xline}, coordinates: {str(self.coordinate)}, trace: {self.trace}>"
 
     @staticmethod
     def from_proto(proto) -> "Trace":
         return Trace(
             trace_header=proto.trace_header,
             inline=proto.iline.value,
-            crossline=proto.xline.value,
+            xline=proto.xline.value,
             trace=proto.trace,
             coordinate=Coordinate.from_proto(proto.coordinate),
         )
 
 
+class RangeInclusive:
+    """Represents an inclusive range of inlines/xlines or depth coordinates.
+
+    Attributes:
+        start: The first linenumber encompassed by the range
+        stop: The last linenumber encompassed by the range
+        step: The distance between linenumbers"""
+
+    start: int
+    stop: int
+    step: int
+
+    def __init__(self, start: int, stop: int, step: Optional[int] = 1):
+        if step == 0:
+            raise ValueError("RangeInclusive(): step must be nonzero")
+        self.start = start
+        self.stop = start + ((stop - start) // step) * step
+        self.step = step
+
+    @classmethod
+    def from_linerange(cls, linerange: LineRange) -> "RangeInclusive":
+        """Construct a RangeInclusive from a (start, stop) or (start, stop, step) tuple"""
+        if len(linerange) == 2:
+            return cls(linerange[0], linerange[1], 1)
+        elif len(linerange) == 3:
+            return cls(linerange[0], linerange[1], linerange[2])
+        else:
+            raise ValueError("Linerange must have 2 or 3 elements")
+
+    @classmethod
+    def widest(cls, ranges: Iterable[Tuple[int, int]]) -> "RangeInclusive":
+        """Finds the widest range among the (min, max) tuples in ranges, and returns the result as a RangeInclusive"""
+        min_start = None
+        max_stop = None
+        for (start, stop) in ranges:
+            if min_start is None or start < min_start:
+                min_start = start
+            if max_stop is None or stop > max_stop:
+                max_stop = stop
+        if min_start is None or max_stop is None:
+            raise ValueError("Empty range")
+        return RangeInclusive(min_start, max_stop)
+
+    def index(self, line: int) -> int:
+        """Compute the index of a given linenumber in this range"""
+        if line < self.start or line > self.stop:
+            raise ValueError(f"line {line} out of range")
+        if (line - self.start) % self.step != 0:
+            raise ValueError(f"line {line} incompatible with step {self.step} starting from {self.start}")
+        return (line - self.start) // self.step
+
+    def to_linerange(self) -> LineRange:
+        """Return a (start, stop) or a (start, stop, step) tuple"""
+        if self.step == 1:
+            return (self.start, self.stop)
+        else:
+            return (self.start, self.stop, self.step)
+
+    def __len__(self) -> int:
+        """Return the number of lines described by this range"""
+        return (self.stop - self.start) // self.step + 1
+
+    def __iter__(self):
+        cur = self.start
+        while cur <= self.stop:
+            yield cur
+            cur += self.step
+
+    def __contains__(self, line: int) -> bool:
+        return line >= self.start and line <= self.stop and (line - self.start) % self.step == 0
+
+    def __repr__(self):
+        return f"RangeInclusive({self.start}, {self.stop}, {self.step})"
+
+    def __eq__(self, other):
+        return self.start == other.start and self.stop == other.stop and self.step == other.step
+
+
 class VolumeDef:
-    """Represents a VolumeDef, a method to describe an inline/crossline grid. Refer to :ref:`VolumeDef Overview` for more information."""
+    """Represents a VolumeDef, a method to describe an inline/xline grid. Refer to :ref:`VolumeDef Overview` for more information."""
 
     json: str
 
     def __init__(self, json_payload):
         self.json = json_payload
         self.parsed = json.loads(json_payload)
+        if self.parsed["version"] > 102:
+            raise Exception(
+                f'The specified volumedef version {self.parsed["version"]} is not supported by this version of the Cognite SDK.'
+            )
+        if "direction" in self.parsed:
+            if self.parsed["direction"] == "inline":
+                self.direction = Direction.INLINE
+            elif self.parsed["direction"] == "crossline":
+                self.direction = Direction.XLINE
+            else:
+                raise Exception(f'Unknown direction "{direction}"')
+        else:
+            self.direction = Direction.INLINE
 
     def __repr__(self):
         return f"VolumeDef<{self.json}>"
@@ -108,6 +201,40 @@ class VolumeDef:
     def from_proto(proto) -> "VolumeDef":
         return VolumeDef(json_payload=proto.json)
 
+    @staticmethod
+    def _get_range_step(method) -> int:
+        """Given a range method, find the step size"""
+        range_ = method["range"]
+        if len(range_) == 3:
+            return range_[2]
+        else:
+            # Accounting for negative ranges
+            if range_[0] < range_[1]:
+                return 1
+            else:
+                return -1
+
+    @staticmethod
+    def _iterate_method(method) -> Iterable[int]:
+        """Given a volumedef method, iterate over each minor number in it."""
+        if "list" in method:
+            return method["list"]
+        elif "range" in method:
+            lower, upper = method["range"][0:2]
+            step = VolumeDef._get_range_step(method)
+            range_ = [x for x in range(lower, upper + step, step)]
+            if range_[-1] > upper:
+                # Needed because range() will go up to one element beyond upper+step if it isn't cleanly divisible.
+                range_.pop()
+            return range_
+
+    @staticmethod
+    def _iterate_methods(methods) -> Iterable[int]:
+        "Given a list of volumedef methods, iterate over each number described by it"
+        for method in methods:
+            for value in VolumeDef._iterate_method(method):
+                yield value
+
     @staticmethod
     def _get_method_size(method) -> int:
         """Given a volumedef method, calculate the number of traces within it"""
@@ -116,20 +243,31 @@ class VolumeDef:
             c += len(method["list"])
         elif "range" in method:
             range_ = method["range"]
-            if len(range_) == 3:
-                step = range_[2]
-            else:
-                # Accounting for negative ranges
-                if range_[0] < range_[1]:
-                    step = 1
-                else:
-                    step = -1
+            step = VolumeDef._get_range_step(method)
             c += (range_[1] - range_[0]) // step + 1
         else:
             raise Exception(f"The specified method {method} was not recognized.")
 
         return c
 
+    @staticmethod
+    def _get_method_range(method) -> Tuple[int, int]:
+        """Given a method, find the minimum and maximum coordinates within it"""
+        if "list" in method:
+            minimum = min(method["list"])
+            maximum = max(method["list"])
+            return (minimum, maximum)
+        elif "range" in method:
+            lower, upper = method["range"][0:2]
+            step = VolumeDef._get_range_step(method)
+
+            # Find highest multiple of step not greater than upper
+            end = lower + ((upper - lower) // step) * step
+
+            return tuple(sorted((lower, end)))  # Sort to handle negative-step volumedefs
+        else:
+            raise Exception(f"The specified method {method} was not recognized.")
+
     def count_line_traces(self) -> Mapping[int, int]:
         """Count the number of traces by line.
 
@@ -156,6 +294,130 @@ class VolumeDef:
             c += val
         return c
 
+    def get_major_range(self) -> Mapping[int, Tuple[int, int]]:
+        """Return the min & max range of traces by major line, i.e. a mapping from major number to minor range.
+
+        Returns:
+            Mapping[int, Tuple[int, int]]: A mapping of line number to minimum and maximum value for that line.
+        """
+        output = {}
+        for key, methods in self.parsed["lines"].items():
+            lower, upper = None, None
+            for x, y in map(VolumeDef._get_method_range, methods):
+                if lower is None or x < lower:
+                    lower = x
+                if upper is None or upper < y:
+                    upper = y
+            output[int(key)] = (lower, upper)
+        return output
+
+    def get_minor_range(self) -> Mapping[int, Tuple[int, int]]:
+        """Return the min & max range of traces by minor line, i.e. a mapping from minor number to major range.
+
+        Returns:
+            Mapping[int, Tuple[int, int]]: A mapping of line number to minimum and maximum value for that line.
+        """
+        output = {}
+        for key, methods in self.parsed["lines"].items():
+            for minor in VolumeDef._iterate_methods(methods):
+                lower, upper = output.setdefault(minor, (None, None))
+                if lower is None or int(key) < lower:
+                    lower = int(key)
+                if upper is None or int(key) > upper:
+                    upper = int(key)
+                output[minor] = (lower, upper)
+        return output
+
+    def get_inline_range(self) -> Mapping[int, Tuple[int, int]]:
+        """Return the min & max range of traces by inline. That is, the mapping will be from inline numbers to xline numbers.
+
+        Returns:
+            Mapping[int, Tuple[int, int]]: A mapping of line number to minimum and maximum value for that line.
+        """
+        if self.direction == Direction.INLINE:
+            return self.get_major_range()
+        else:
+            return self.get_minor_range()
+
+    def get_xline_range(self) -> Mapping[int, Tuple[int, int]]:
+        """Return the min & max range of traces by xline. That is, the mapping will be from xline numbers to inline numbers.
+
+        Returns:
+            Mapping[int, Tuple[int, int]]: A mapping of line number to minimum and maximum value for that line.
+        """
+        if self.direction == Direction.XLINE:
+            return self.get_major_range()
+        else:
+            return self.get_minor_range()
+
+    def common_major_range(self) -> RangeInclusive:
+        """Compute the range with the largest step size encompassing all major lines in this VolumeDef
+
+        Returns:
+            RangeInclusive: Range describing the minimum, maximum, and step size of line numbers
+        """
+        majors = [int(major) for major in self.parsed["lines"].keys()]
+        min_major = min(majors)
+        max_major = max(majors)
+        step = max_major - min_major
+        for major in majors:
+            if major < min_major or major > max_major:
+                continue
+            step = math.gcd(step, major - min_major)
+            if step == 1:
+                break
+        return RangeInclusive(min_major, max_major, step)
+
+    def common_minor_range(self) -> RangeInclusive:
+        """Compute the range with the largest step size encompassing all minor lines in this VolumeDef
+
+        Returns:
+            RangeInclusive: Range describing the minimum, maximum, and step size of line numbers
+        """
+        ranges = self.get_major_range().values()
+        min_minor = min(r[0] for r in ranges)
+        max_minor = max(r[1] for r in ranges)
+        step = max_minor - min_minor
+        for methods in self.parsed["lines"].values():
+            # Let's be stupid for now and just check all lines
+            # This can be optimized by not iterating through the ranges
+            for minor in self._iterate_methods(methods):
+                step = math.gcd(step, minor - min_minor)
+                if step == 1:
+                    return RangeInclusive(min_minor, max_minor, step)
+        return RangeInclusive(min_minor, max_minor, step)
+
+    def common_inline_range(self) -> RangeInclusive:
+        """Compute the range with the largest step size encompassing all inlines in this VolumeDef
+
+        Returns:
+            RangeInclusive: Range describing the minimum, maximum, and step size of line numbers
+        """
+        if self.direction == Direction.INLINE:
+            return self.common_major_range()
+        else:
+            return self.common_minor_range()
+
+    def common_xline_range(self) -> RangeInclusive:
+        """Compute the range with the largest step size encompassing all xlines in this VolumeDef
+
+        Returns:
+            RangeInclusive: Range describing the minimum, maximum, and step size of line numbers
+        """
+        if self.direction == Direction.XLINE:
+            return self.common_major_range()
+        else:
+            return self.common_minor_range()
+
+    def max_ranges(self) -> Tuple[RangeInclusive, RangeInclusive]:
+        ranges = self.get_major_range()
+        major = RangeInclusive(min(ranges.keys()), max(ranges.keys()))
+        minor = RangeInclusive.widest(ranges.values())
+        if self.direction == Direction.INLINE:
+            return (major, minor)
+        else:
+            return (minor, major)
+
 
 class File:
     """Represents a raw SEGY file.
@@ -427,7 +689,7 @@ class SeismicStore:
         metadata (Mapping[str, str]): Any custom-defined metadata
         ingested_file (Optional[:py:class:`File`]): If present, the file this SeismicStore was ingested from
         inline_volume_def (Optional[:py:class:`VolumeDef`]): The inline-major volume definition for this seismic store.
-        crossline_volume_def (Optional[:py:class:`VolumeDef`]): The crossline-major volume definition for this seismic store.
+        xline_volume_def (Optional[:py:class:`VolumeDef`]): The xline-major volume definition for this seismic store.
         coverage (Optional[:py:class:`Geometry`]): If present, the coverage geometry for this seismic store
         text_header (Optional[:py:class:`TextHeader`]): If present, the text header for this seismic store
         binary_header (Optional[:py:class:`BinaryHeader`]): If present, the binary header for this seismic store
@@ -441,7 +703,7 @@ class SeismicStore:
     metadata: Mapping[str, str]
     ingested_file: Optional[File]
     inline_volume_def: Optional[VolumeDef]
-    crossline_volume_def: Optional[VolumeDef]
+    xline_volume_def: Optional[VolumeDef]
     coverage: Geometry
     text_header: Optional[TextHeader]
     binary_header: Optional[BinaryHeader]
@@ -457,7 +719,7 @@ class SeismicStore:
         metadata,
         ingested_file,
         inline_volume_def,
-        crossline_volume_def,
+        xline_volume_def,
         coverage,
         text_header,
         binary_header,
@@ -470,7 +732,7 @@ class SeismicStore:
         self.metadata = metadata
         self.ingested_file = ingested_file
         self.inline_volume_def = inline_volume_def
-        self.crossline_volume_def = crossline_volume_def
+        self.xline_volume_def = xline_volume_def
         self.coverage = coverage
         self.text_header = text_header
         self.binary_header = binary_header
@@ -487,14 +749,14 @@ class SeismicStore:
 
         # Only decode protobuf fields that are defined.
         inline_volume_def = None
-        crossline_volume_def = None
+        xline_volume_def = None
         coverage = None
         text_header = None
         binary_header = None
         if proto.HasField("inline_volume_def"):
             inline_volume_def = VolumeDef.from_proto(proto.inline_volume_def)
         if proto.HasField("crossline_volume_def"):
-            crossline_volume_def = VolumeDef.from_proto(proto.crossline_volume_def)
+            xline_volume_def = VolumeDef.from_proto(proto.crossline_volume_def)
         if proto.HasField("coverage"):
             coverage = Geometry.from_proto(proto.coverage)
         if proto.HasField("text_header"):
@@ -510,7 +772,7 @@ class SeismicStore:
             metadata=metadata,
             ingested_file=proto.ingested_file,
             inline_volume_def=inline_volume_def,
-            crossline_volume_def=crossline_volume_def,
+            xline_volume_def=xline_volume_def,
             coverage=coverage,
             text_header=text_header,
             binary_header=binary_header,
@@ -524,17 +786,17 @@ class SeismicLineRange:
     inline_min: int
     inline_max: int
     inline_step: int
-    crossline_min: int
-    crossline_max: int
-    crossline_step: int
+    xline_min: int
+    xline_max: int
+    xline_step: int
 
-    def __init__(self, inline_min, inline_max, inline_step, crossline_min, crossline_max, crossline_step):
+    def __init__(self, inline_min, inline_max, inline_step, xline_min, xline_max, xline_step):
         self.inline_min = inline_min
         self.inline_max = inline_max
         self.inline_step = inline_step
-        self.crossline_min = crossline_min
-        self.crossline_max = crossline_max
-        self.crossline_step = crossline_step
+        self.xline_min = xline_min
+        self.xline_max = xline_max
+        self.xline_step = xline_step
 
     def __repr__(self):
         return (
@@ -542,9 +804,9 @@ class SeismicLineRange:
             f"inline_min: {self.inline_min}, "
             f"inline_max: {self.inline_max}, "
             f"inline_step: {self.inline_step}, "
-            f"crossline_min: {self.crossline_min}, "
-            f"crossline_max: {self.crossline_max}, "
-            f"crossline_step: {self.crossline_step}>"
+            f"xline_min: {self.xline_min}, "
+            f"xline_max: {self.xline_max}, "
+            f"xline_step: {self.xline_step}>"
         )
 
     @staticmethod
@@ -553,9 +815,9 @@ class SeismicLineRange:
             inline_min=proto.inline.min.value,
             inline_max=proto.inline.max.value,
             inline_step=proto.inline.step.value,
-            crossline_min=proto.crossline.min.value,
-            crossline_max=proto.crossline.max.value,
-            crossline_step=proto.crossline.step.value,
+            xline_min=proto.crossline.min.value,
+            xline_max=proto.crossline.max.value,
+            xline_step=proto.crossline.step.value,
         )
 
 
@@ -697,14 +959,16 @@ class Survey:
     seismic_ids: Optional[List[int]]
     seismic_store_ids: Optional[List[int]]
     metadata: Optional[Mapping[str, str]]
+    coverage: Optional[Geometry]
 
-    def __init__(self, id, external_id, name, seismic_ids, seismic_store_ids, metadata):
+    def __init__(self, id, external_id, name, seismic_ids, seismic_store_ids, metadata, coverage):
         self.id = id
         self.external_id = external_id
         self.name = name
         self.seismic_ids = seismic_ids
         self.seismic_store_ids = seismic_store_ids
         self.metadata = metadata
+        self.coverage = coverage
 
     @staticmethod
     def from_proto(proto) -> "Survey":
@@ -714,8 +978,9 @@ class Survey:
         metadata = proto.survey.metadata if hasattr(proto.survey, "metadata") else None
         seismic_ids = proto.seismic_ids if hasattr(proto, "seismic_ids") else None
         seismic_store_ids = proto.seismic_store_ids if hasattr(proto, "seismic_store_ids") else None
+        coverage = Geometry.from_proto(proto.coverage) if proto.HasField("coverage") else None
 
-        return Survey(survey_id, external_id, name, seismic_ids, seismic_store_ids, metadata)
+        return Survey(survey_id, external_id, name, seismic_ids, seismic_store_ids, metadata, coverage)
 
     def __repr__(self):
         return "%s {\n%s\n}" % (type(self).__name__, "\n".join("%s = %s" % item for item in vars(self).items()))
